package calculator;

/**
*
* @author andreas
*/
public class NumeroBinario {

	// Caracteres que represnetas los n&uacute;meros binarios.
	private final static char ZERO_CHAR = '0';
	private final static char ONE_CHAR = '1';
	
	// Referencia a un String que contiene un n&uacute;mero binario
	// As no pueden decir que hacemos trampa!
	private String numero;

	// Validar que el n&uacute;mero sea binario
	private boolean validaNumeroBinario(String numero) {

		final String REGEX = "[01]+";
		return numero.matches(REGEX);

	}

	// Establecer el n&uacute;mero
	public void setNumero(String numero) {
		this.numero = numero;
	}

	// Obtener el n&uacute;mero
	public String getNumero() {
		return numero;
	}

	// Constructor de NumeroBinario
	public NumeroBinario(String numero) {

		// Hay que validar que sea un n&uacute;mero binario!!!

		if (!this.validaNumeroBinario(numero)) {

			throw new IllegalArgumentException("El nmero no es binario");

		}

		this.numero = numero;

	}

	// M&eacute;todo que rellena con ceros por la izquierda
	private String fillWithLeadingZeros(String numero, int longitud) {

		if (longitud == numero.length()) {

			return numero;

		}

		int missing = longitud - numero.length();

		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < missing; i++) {

			sb.append(ZERO_CHAR);

		}

		sb.append(numero);

		return sb.toString();

	}

	// M&eacute;todo que rellena con ceros por la derecha
	private String fillWithTrailingZeros(String numero, int longitud) {

		if (longitud == numero.length()) {

			return numero;

		}

		StringBuilder sb = new StringBuilder();
		sb.append(numero);

		for (int i = 0; i < longitud; i++) {

			sb.append(ZERO_CHAR);

		}

		return sb.toString();

	}

	// M&eacute;todo que suma el n&uacute;mero binario actual con el pasado como argumento
	public NumeroBinario sumar(NumeroBinario otroNumero) {

		int maxLength = Math.max(this.numero.length(), otroNumero.numero.length());

		String zfNumero = fillWithLeadingZeros(this.numero, maxLength);
		String zfOtroNumero = fillWithLeadingZeros(otroNumero.numero, maxLength);

		char carry = ZERO_CHAR;
		char[] result = new char[maxLength + 1];

		for (int i = maxLength - 1; i >= 0; i--) {

			char zfNumeroCharAtI = zfNumero.charAt(i);
			char zfOtroNumeroCharAtI = zfOtroNumero.charAt(i);

			if (zfNumeroCharAtI == zfOtroNumeroCharAtI) {

				if (ZERO_CHAR == zfNumeroCharAtI) {

					if (ONE_CHAR == carry) {
						result[i + 1] = ONE_CHAR;
						carry = ZERO_CHAR;
					} else {
						result[i + 1] = ZERO_CHAR;
						carry = ZERO_CHAR;
					}

				} else {

					if (ONE_CHAR == carry) {
						result[i + 1] = ONE_CHAR;
						carry = ONE_CHAR;
					} else {
						result[i + 1] = ZERO_CHAR;
						carry = ONE_CHAR;
					}

				}

			} else {

				if (ONE_CHAR == carry) {
					result[i + 1] = ZERO_CHAR;
					carry = ONE_CHAR;
				} else {
					result[i + 1] = ONE_CHAR;
					carry = ZERO_CHAR;
				}

			}

		}

		result[0] = carry;

		return new NumeroBinario(new String(result).trim());
	}

	// M&eacute;todo que resta el n&uacute;mero binario actual con el pasado como argumento
	public NumeroBinario restar(NumeroBinario otroNumero) {

		int maxLength = Math.max(this.numero.length(), otroNumero.numero.length());

		String zfOtroNumero = fillWithLeadingZeros(otroNumero.numero, maxLength);

		char[] complement = new char[maxLength];

		for (int i = 0; i < maxLength; i++) {

			complement[i] = ZERO_CHAR == zfOtroNumero.charAt(i) ? ONE_CHAR : ZERO_CHAR;

		}

		NumeroBinario one = new NumeroBinario(Character.toString(ONE_CHAR));
		NumeroBinario tmp = new NumeroBinario(new String(complement));

		NumeroBinario result = tmp.sumar(one);

		result = this.sumar(result);

		result.numero = result.getNumero().substring(2);

		return result;

	}

	
	// M&eacute;todo que multiplica el n&uacute;mero binario actual con el pasado como argumento
	public NumeroBinario multiplicar(NumeroBinario otroNumero) {

		int maxLength = otroNumero.numero.length();

		NumeroBinario sum = new NumeroBinario(Character.toString(ZERO_CHAR));
		int limit = maxLength - 1;

		for (int i = limit; i >= 0; i--) {

			if (otroNumero.numero.charAt(i) == ONE_CHAR) {

				String trail = this.fillWithTrailingZeros(this.numero, limit - i);
				NumeroBinario tmp = new NumeroBinario(trail);
				sum = sum.sumar(tmp);
				sum.setNumero(sum.numero.substring(1));

			}

		}

		return sum;

	}

}
